@@ -87,12 +87,10 @@ commit_loose_object_trusted (OstreeRepo        *self,
           goto out;
         }
               
-      /* Sadly we can't use at-relative API for symlink xattrs
-       * because there's no lsetxattrat.
-       */
       if (xattrs != NULL)
         {
-          if (!gs_file_set_all_xattrs (temp_file, xattrs, cancellable, error))
+          if (!gs_dfd_and_name_set_all_xattrs (self->tmp_dir_fd, temp_filename,
+                                               xattrs, cancellable, error))
             goto out;
         }
     }
@@ -157,10 +155,13 @@ commit_loose_object_trusted (OstreeRepo        *self,
       /* Ensure that in case of a power cut, these files have the data we
        * want.   See http://lwn.net/Articles/322823/
        */
-      if (fsync (fd) == -1)
+      if (!self->disable_fsync)
         {
-          ot_util_set_error_from_errno (error, errno);
-          goto out;
+          if (fsync (fd) == -1)
+            {
+              ot_util_set_error_from_errno (error, errno);
+              goto out;
+            }
         }
           
       if (!g_output_stream_close (temp_out, cancellable, error))
@@ -1611,6 +1612,12 @@ struct OstreeRepoCommitModifier {
   OstreeRepoCommitFilter filter;
   gpointer user_data;
   GDestroyNotify destroy_notify;
+
+  OstreeRepoCommitModifierXattrCallback xattr_callback;
+  GDestroyNotify xattr_destroy;
+  gpointer xattr_user_data;
+
+  OstreeSePolicy *sepolicy;
 };
 
 OstreeRepoCommitFilterResult
@@ -1636,21 +1643,10 @@ _ostree_repo_commit_modifier_apply (OstreeRepo               *self,
   return result;
 }
 
-static gboolean
-apply_commit_filter (OstreeRepo               *self,
-                     OstreeRepoCommitModifier *modifier,
-                     GPtrArray                *path,
-                     GFileInfo                *file_info,
-                     GFileInfo               **out_modified_info)
+static char *
+ptrarray_path_join (GPtrArray  *path)
 {
   GString *path_buf;
-  OstreeRepoCommitFilterResult result;
-
-  if (modifier == NULL || modifier->filter == NULL)
-    {
-      *out_modified_info = g_object_ref (file_info);
-      return OSTREE_REPO_COMMIT_FILTER_ALLOW;
-    }
 
   path_buf = g_string_new ("");
 
@@ -1668,10 +1664,84 @@ apply_commit_filter (OstreeRepo               *self,
         }
     }
 
-  result = _ostree_repo_commit_modifier_apply (self, modifier, path_buf->str, file_info, out_modified_info);
+  return g_string_free (path_buf, FALSE);
+}
 
-  g_string_free (path_buf, TRUE);
-  return result;
+static gboolean
+apply_commit_filter (OstreeRepo               *self,
+                     OstreeRepoCommitModifier *modifier,
+                     const char               *relpath,
+                     GFileInfo                *file_info,
+                     GFileInfo               **out_modified_info)
+{
+  if (modifier == NULL || modifier->filter == NULL)
+    {
+      *out_modified_info = g_object_ref (file_info);
+      return OSTREE_REPO_COMMIT_FILTER_ALLOW;
+    }
+
+  return _ostree_repo_commit_modifier_apply (self, modifier, relpath, file_info, out_modified_info);
+}
+
+static gboolean
+get_modified_xattrs (OstreeRepo                       *self,
+                     OstreeRepoCommitModifier         *modifier,
+                     const char                       *relpath,
+                     GFileInfo                        *file_info,
+                     GFile                            *path,
+                     GVariant                        **out_xattrs,
+                     GCancellable                     *cancellable,
+                     GError                          **error)
+{
+  gboolean ret = FALSE;
+  gs_unref_variant GVariant *ret_xattrs = NULL;
+
+  if (modifier && modifier->xattr_callback)
+    {
+      ret_xattrs = modifier->xattr_callback (self, relpath, file_info,
+                                             modifier->xattr_user_data);
+    }
+  else if (!(modifier && (modifier->flags & OSTREE_REPO_COMMIT_MODIFIER_FLAGS_SKIP_XATTRS) > 0))
+    {
+      if (!gs_file_get_all_xattrs (path, &ret_xattrs, cancellable, error))
+        goto out;
+    }
+
+  if (modifier && modifier->sepolicy)
+    {
+      gs_free char *label = NULL;
+
+      if (!ostree_sepolicy_get_label (modifier->sepolicy, relpath,
+                                      g_file_info_get_attribute_uint32 (file_info, "unix::mode"),
+                                      &label, cancellable, error))
+        goto out;
+
+      if (label)
+        {
+          GVariantBuilder *builder;
+
+          if (ret_xattrs)
+            builder = ot_util_variant_builder_from_variant (ret_xattrs,
+                                                            G_VARIANT_TYPE ("a(ayay)"));
+          else
+            builder = g_variant_builder_new (G_VARIANT_TYPE ("a(ayay)"));
+
+          g_variant_builder_add_value (builder,
+                                       g_variant_new ("(@ay@ay)",
+                                                      g_variant_new_bytestring ("security.selinux"),
+                                                      g_variant_new_bytestring (label)));
+          if (ret_xattrs)
+            g_variant_unref (ret_xattrs);
+
+          ret_xattrs = g_variant_builder_end (builder);
+          g_variant_ref_sink (ret_xattrs);
+        }
+    }
+  
+  ret = TRUE;
+  gs_transfer_out_value (out_xattrs, &ret_xattrs);
+ out:
+  return ret;
 }
 
 static gboolean
@@ -1726,6 +1796,7 @@ write_directory_to_mtree_internal (OstreeRepo                  *self,
       gs_unref_variant GVariant *xattrs = NULL;
       gs_free guchar *child_file_csum = NULL;
       gs_free char *tmp_checksum = NULL;
+      gs_free char *relpath = NULL;
 
       child_info = g_file_query_info (dir, OSTREE_GIO_FAST_QUERYINFO,
                                       G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
@@ -1733,16 +1804,18 @@ write_directory_to_mtree_internal (OstreeRepo                  *self,
       if (!child_info)
         goto out;
 
-      filter_result = apply_commit_filter (self, modifier, path, child_info, &modified_info);
+      if (modifier != NULL)
+        relpath = ptrarray_path_join (path);
+
+      filter_result = apply_commit_filter (self, modifier, relpath, child_info, &modified_info);
 
       if (filter_result == OSTREE_REPO_COMMIT_FILTER_ALLOW)
         {
           g_debug ("Adding: %s", gs_file_get_path_cached (dir));
-          if (!(modifier && (modifier->flags & OSTREE_REPO_COMMIT_MODIFIER_FLAGS_SKIP_XATTRS) > 0))
-            {
-              if (!gs_file_get_all_xattrs (dir, &xattrs, cancellable, error))
-                goto out;
-            }
+          if (!get_modified_xattrs (self, modifier, relpath, child_info, dir,
+                                    &xattrs,
+                                    cancellable, error))
+            goto out;
 
           if (!_ostree_repo_write_directory_meta (self, modified_info, xattrs, &child_file_csum,
                                                   cancellable, error))
@@ -1771,6 +1844,7 @@ write_directory_to_mtree_internal (OstreeRepo                  *self,
           gs_unref_object GFile *child = NULL;
           gs_unref_object GFileInfo *modified_info = NULL;
           gs_unref_object OstreeMutableTree *child_mtree = NULL;
+          gs_free char *child_relpath = NULL;
           const char *name;
 
           if (!gs_file_enumerator_iterate (dir_enum, &child_info, NULL,
@@ -1781,7 +1855,11 @@ write_directory_to_mtree_internal (OstreeRepo                  *self,
 
           name = g_file_info_get_name (child_info);
           g_ptr_array_add (path, (char*)name);
-          filter_result = apply_commit_filter (self, modifier, path, child_info, &modified_info);
+
+          if (modifier != NULL)
+            child_relpath = ptrarray_path_join (path);
+
+          filter_result = apply_commit_filter (self, modifier, child_relpath, child_info, &modified_info);
 
           if (filter_result == OSTREE_REPO_COMMIT_FILTER_ALLOW)
             {
@@ -1842,19 +1920,18 @@ write_directory_to_mtree_internal (OstreeRepo                  *self,
                     }
                   else
                     {
-                     if (g_file_info_get_file_type (modified_info) == G_FILE_TYPE_REGULAR)
+                      if (g_file_info_get_file_type (modified_info) == G_FILE_TYPE_REGULAR)
                         {
                           file_input = (GInputStream*)g_file_read (child, cancellable, error);
                           if (!file_input)
                             goto out;
                         }
 
-                      if (!(modifier && (modifier->flags & OSTREE_REPO_COMMIT_MODIFIER_FLAGS_SKIP_XATTRS) > 0))
-                        {
-                          g_clear_pointer (&xattrs, (GDestroyNotify) g_variant_unref);
-                          if (!gs_file_get_all_xattrs (child, &xattrs, cancellable, error))
-                            goto out;
-                        }
+                      if (!get_modified_xattrs (self, modifier,
+                                                child_relpath, child_info, child,
+                                                &xattrs,
+                                                cancellable, error))
+                        goto out;
 
                       if (!ostree_raw_file_to_content_stream (file_input,
                                                               modified_info, xattrs,
@@ -2051,10 +2128,60 @@ ostree_repo_commit_modifier_unref (OstreeRepoCommitModifier *modifier)
   if (modifier->destroy_notify)
     modifier->destroy_notify (modifier->user_data);
 
+  if (modifier->xattr_destroy)
+    modifier->xattr_destroy (modifier->xattr_user_data);
+
+  g_clear_object (&modifier->sepolicy);
+
   g_free (modifier);
   return;
 }
 
+/**
+ * ostree_repo_commit_modifier_set_xattr_callback:
+ * @modifier: An #OstreeRepoCommitModifier
+ * @callback: Function to be invoked, should return extended attributes for path
+ * @destroy: Destroy notification
+ * @user_data: Data for @callback:
+ *
+ * If set, this function should return extended attributes to use for
+ * the given path.  This is useful for things like ACLs and SELinux,
+ * where a build system can label the files as it's committing to the
+ * repository.
+ */
+void
+ostree_repo_commit_modifier_set_xattr_callback (OstreeRepoCommitModifier  *modifier,
+                                                OstreeRepoCommitModifierXattrCallback  callback,
+                                                GDestroyNotify                         destroy,
+                                                gpointer                               user_data)
+{
+  modifier->xattr_callback = callback;
+  modifier->xattr_destroy = destroy;
+  modifier->xattr_user_data = user_data;
+}
+
+/**
+ * ostree_repo_commit_modifier_set_sepolicy:
+ * @modifier: An #OstreeRepoCommitModifier
+ * @sepolicy: (allow-none): Policy to use for labeling
+ *
+ * If @policy is non-%NULL, use it to look up labels to use for
+ * "security.selinux" extended attributes.
+ *
+ * Note that any policy specified this way operates in addition to any
+ * extended attributes provided via
+ * ostree_repo_commit_modifier_set_xattr_callback().  However if both
+ * specify a value for "security.selinux", then the one from the
+ * policy wins.
+ */
+void
+ostree_repo_commit_modifier_set_sepolicy (OstreeRepoCommitModifier              *modifier,
+                                          OstreeSePolicy                        *sepolicy)
+{
+  g_clear_object (&modifier->sepolicy);
+  modifier->sepolicy = sepolicy ? g_object_ref (sepolicy) : NULL;
+}
+
 G_DEFINE_BOXED_TYPE(OstreeRepoCommitModifier, ostree_repo_commit_modifier,
                     ostree_repo_commit_modifier_ref,
                     ostree_repo_commit_modifier_unref);
