@@ -42,12 +42,6 @@ struct _OstreeRepoFileClass
 
 GType   ostree_repo_file_get_type (void) G_GNUC_CONST;
 
-GFile * ostree_repo_file_new_root (OstreeRepo  *repo,
-                                    const char  *commit);
-
-GFile * ostree_repo_file_new_child (OstreeRepoFile *parent,
-                                     const char  *name);
-
 gboolean ostree_repo_file_ensure_resolved (OstreeRepoFile  *self,
                                             GError         **error);
 
@@ -65,18 +59,14 @@ void ostree_repo_file_tree_set_metadata (OstreeRepoFile *self,
                                           const char     *checksum,
                                           GVariant       *metadata);
 
-void ostree_repo_file_tree_set_content_checksum (OstreeRepoFile  *self,
-                                                  const char      *checksum);
-const char *ostree_repo_file_tree_get_content_checksum (OstreeRepoFile  *self);
+const char *ostree_repo_file_tree_get_contents_checksum (OstreeRepoFile  *self);
+const char *ostree_repo_file_tree_get_metadata_checksum (OstreeRepoFile  *self);
 
-gboolean ostree_repo_file_is_tree (OstreeRepoFile  *self);
+GVariant *ostree_repo_file_tree_get_contents (OstreeRepoFile *self);
+GVariant *ostree_repo_file_tree_get_metadata (OstreeRepoFile *self);
 
 const char * ostree_repo_file_get_checksum (OstreeRepoFile  *self);
 
-const char * ostree_repo_file_get_commit (OstreeRepoFile  *self);
-
-GFile *ostree_repo_file_nontree_get_local (OstreeRepoFile  *self);
-
 int     ostree_repo_file_tree_find_child  (OstreeRepoFile  *self,
                                             const char      *name,
                                             gboolean        *is_dir,
@@ -90,8 +80,5 @@ gboolean ostree_repo_file_tree_query_child (OstreeRepoFile  *self,
                                              GCancellable    *cancellable,
                                              GError         **error);
 
-GVariant *ostree_repo_file_tree_get_contents (OstreeRepoFile  *self);
-GVariant *ostree_repo_file_tree_get_metadata (OstreeRepoFile  *self);
-
 G_END_DECLS
 
