@@ -35,20 +35,6 @@
 
 #ifdef HAVE_LIBARCHIVE
 
-static GFileInfo *
-create_modified_file_info (GFileInfo               *info,
-                           OstreeRepoCommitModifier *modifier)
-{
-  GFileInfo *ret;
-
-  if (!modifier)
-    return (GFileInfo*)g_object_ref (info);
-
-  ret = g_file_info_dup (info);
-  
-  return ret;
-}
-
 static void
 propagate_libarchive_error (GError      **error,
                             struct archive *a)
@@ -58,10 +44,11 @@ propagate_libarchive_error (GError      **error,
 }
 
 static GFileInfo *
-file_info_from_archive_entry_and_modifier (struct archive_entry  *entry,
+file_info_from_archive_entry_and_modifier (OstreeRepo *repo,
+                                           struct archive_entry *entry,
                                            OstreeRepoCommitModifier *modifier)
 {
-  GFileInfo *info = g_file_info_new ();
+  gs_unref_object GFileInfo *info = g_file_info_new ();
   GFileInfo *modified_info = NULL;
   const struct stat *st;
   guint32 file_type;
@@ -88,10 +75,10 @@ file_info_from_archive_entry_and_modifier (struct archive_entry  *entry,
       g_file_info_set_attribute_uint32 (info, "unix::rdev", st->st_rdev);
     }
 
-  modified_info = create_modified_file_info (info, modifier);
+  _ostree_repo_commit_modifier_apply (repo, modifier,
+                                      archive_entry_pathname (entry),
+                                      info, &modified_info);
 
-  g_object_unref (info);
-  
   return modified_info;
 }
 
@@ -119,7 +106,7 @@ import_libarchive_entry_file (OstreeRepo           *self,
                                           &file_object_input, &length, cancellable, error))
     goto out;
   
-  if (!ostree_repo_stage_content (self, NULL, file_object_input, length, out_csum,
+  if (!ostree_repo_write_content (self, NULL, file_object_input, length, out_csum,
                                   cancellable, error))
     goto out;
 
@@ -129,7 +116,7 @@ import_libarchive_entry_file (OstreeRepo           *self,
 }
 
 static gboolean
-stage_libarchive_entry_to_mtree (OstreeRepo           *self,
+write_libarchive_entry_to_mtree (OstreeRepo           *self,
                                  OstreeMutableTree    *root,
                                  struct archive       *a,
                                  struct archive_entry *entry,
@@ -230,7 +217,7 @@ stage_libarchive_entry_to_mtree (OstreeRepo           *self,
     }
   else
     {
-      file_info = file_info_from_archive_entry_and_modifier (entry, modifier);
+      file_info = file_info_from_archive_entry_and_modifier (self, entry, modifier);
 
       if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_UNKNOWN)
         {
@@ -242,7 +229,7 @@ stage_libarchive_entry_to_mtree (OstreeRepo           *self,
       if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY)
         {
 
-          if (!_ostree_repo_stage_directory_meta (self, file_info, NULL, &tmp_csum, cancellable, error))
+          if (!_ostree_repo_write_directory_meta (self, file_info, NULL, &tmp_csum, cancellable, error))
             goto out;
 
           if (parent == NULL)
@@ -287,10 +274,23 @@ stage_libarchive_entry_to_mtree (OstreeRepo           *self,
 }
 #endif
                           
+/**
+ * ostree_repo_write_archive_to_mtree:
+ * @self: An #OstreeRepo
+ * @archive: A path to an archive file
+ * @mtree: The #OstreeMutableTree to write to
+ * @modifier: (allow-none): Optional commit modifier
+ * @autocreate_parents: Autocreate parent directories
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Import an archive file @archive into the repository, and write its
+ * file structure to @mtree.
+ */
 gboolean
-ostree_repo_stage_archive_to_mtree (OstreeRepo                *self,
-                                    GFile                     *archive_f,
-                                    OstreeMutableTree         *root,
+ostree_repo_write_archive_to_mtree (OstreeRepo                *self,
+                                    GFile                     *archive,
+                                    OstreeMutableTree         *mtree,
                                     OstreeRepoCommitModifier  *modifier,
                                     gboolean                   autocreate_parents,
                                     GCancellable             *cancellable,
@@ -305,9 +305,13 @@ ostree_repo_stage_archive_to_mtree (OstreeRepo                *self,
   gs_free guchar *tmp_csum = NULL;
 
   a = archive_read_new ();
+#ifdef HAVE_ARCHIVE_READ_SUPPORT_FILTER_ALL
+  archive_read_support_filter_all (a);
+#else
   archive_read_support_compression_all (a);
+#endif
   archive_read_support_format_all (a);
-  if (archive_read_open_filename (a, gs_file_get_path_cached (archive_f), 8192) != ARCHIVE_OK)
+  if (archive_read_open_filename (a, gs_file_get_path_cached (archive), 8192) != ARCHIVE_OK)
     {
       propagate_libarchive_error (error, a);
       goto out;
@@ -332,11 +336,11 @@ ostree_repo_stage_archive_to_mtree (OstreeRepo                *self,
           g_file_info_set_attribute_uint32 (tmp_dir_info, "unix::gid", archive_entry_gid (entry));
           g_file_info_set_attribute_uint32 (tmp_dir_info, "unix::mode", 0755 | S_IFDIR);
           
-          if (!_ostree_repo_stage_directory_meta (self, tmp_dir_info, NULL, &tmp_csum, cancellable, error))
+          if (!_ostree_repo_write_directory_meta (self, tmp_dir_info, NULL, &tmp_csum, cancellable, error))
             goto out;
         }
 
-      if (!stage_libarchive_entry_to_mtree (self, root, a,
+      if (!write_libarchive_entry_to_mtree (self, mtree, a,
                                             entry, modifier,
                                             autocreate_parents ? tmp_csum : NULL,
                                             cancellable, error))
