@@ -64,6 +64,13 @@ maybe_prune_loose_object (OtPruneData        *data,
 
           if (info)
             {
+              if (objtype == OSTREE_OBJECT_TYPE_COMMIT)
+                {
+                  gs_unref_object GFile *detached_metadata =
+                    _ostree_repo_get_commit_metadata_loose_path (data->repo, checksum);
+                  if (!ot_gfile_ensure_unlinked (detached_metadata, cancellable, error))
+                    goto out;
+                }
               if (!gs_file_unlink (objf, cancellable, error))
                 goto out;
               data->freed_bytes += g_file_info_get_size (info);
@@ -127,7 +134,6 @@ ostree_repo_prune (OstreeRepo        *self,
   gpointer key, value;
   gs_unref_hashtable GHashTable *objects = NULL;
   gs_unref_hashtable GHashTable *all_refs = NULL;
-  gs_free char *formatted_freed_size = NULL;
   OtPruneData data = { 0, };
   gboolean refs_only = flags & OSTREE_REPO_PRUNE_FLAGS_REFS_ONLY;
 
@@ -146,7 +152,7 @@ ostree_repo_prune (OstreeRepo        *self,
         {
           const char *checksum = value;
           
-          if (!ostree_repo_traverse_commit (self, checksum, depth, data.reachable,
+          if (!ostree_repo_traverse_commit_union (self, checksum, depth, data.reachable,
                                             cancellable, error))
             goto out;
         }
@@ -170,8 +176,8 @@ ostree_repo_prune (OstreeRepo        *self,
           if (objtype != OSTREE_OBJECT_TYPE_COMMIT)
             continue;
           
-          if (!ostree_repo_traverse_commit (self, checksum, depth, data.reachable,
-                                            cancellable, error))
+          if (!ostree_repo_traverse_commit_union (self, checksum, depth, data.reachable,
+                                                  cancellable, error))
             goto out;
         }
     }
