@@ -99,6 +99,7 @@ ostree_repo_finalize (GObject *object)
   g_clear_object (&self->objects_dir);
   if (self->objects_dir_fd != -1)
     (void) close (self->objects_dir_fd);
+  g_clear_object (&self->deltas_dir);
   g_clear_object (&self->uncompressed_objects_dir);
   if (self->uncompressed_objects_dir_fd != -1)
     (void) close (self->uncompressed_objects_dir_fd);
@@ -175,6 +176,8 @@ ostree_repo_constructed (GObject *object)
 
   self->objects_dir = g_file_get_child (self->repodir, "objects");
   self->uncompressed_objects_dir = g_file_resolve_relative_path (self->repodir, "uncompressed-objects-cache/objects");
+  self->deltas_dir = g_file_get_child (self->repodir, "deltas");
+  self->uncompressed_objects_dir = g_file_get_child (self->repodir, "uncompressed-objects-cache");
   self->remote_cache_dir = g_file_get_child (self->repodir, "remote-cache");
   self->config_file = g_file_get_child (self->repodir, "config");
 
@@ -552,6 +555,24 @@ ostree_repo_open (OstreeRepo    *self,
   return ret;
 }
 
+/**
+ * ostree_repo_set_disable_fsync:
+ * @self: An #OstreeRepo
+ * @disable_fsync: If %TRUE, do not fsync
+ *
+ * Disable requests to fsync() to stable storage during commits.  This
+ * option should only be used by build system tools which are creating
+ * disposable virtual machines, or have higher level mechanisms for
+ * ensuring data consistency.
+ */
+void
+ostree_repo_set_disable_fsync (OstreeRepo    *self,
+                               gboolean       disable_fsync)
+{
+  self->disable_fsync = disable_fsync;
+}
+
+
 /**
  * ostree_repo_get_path:
  * @self:
@@ -1549,14 +1570,23 @@ ostree_repo_sign_commit (OstreeRepo     *self,
     }
 
   info = gpgme_ctx_get_engine_info (context);
+
+  if ((err = gpgme_set_protocol (context, GPGME_PROTOCOL_OpenPGP)) !=
+      GPG_ERR_NO_ERROR)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Unable to set gpg protocol");
+      goto out;
+    }
   
   if (homedir != NULL)
     {
-      if ((err = gpgme_ctx_set_engine_info (context, info->protocol, info->file_name, homedir))
+      if ((err = gpgme_ctx_set_engine_info (context, info->protocol, NULL, homedir))
           != GPG_ERR_NO_ERROR)
         {
           g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                       "Unable to set gpg homedir");
+                       "Unable to set gpg homedir to '%s'",
+                       homedir);
           goto out;
         }
     }
@@ -1565,7 +1595,8 @@ ostree_repo_sign_commit (OstreeRepo     *self,
   if ((err = gpgme_get_key (context, key_id, &key, 1)) != GPG_ERR_NO_ERROR)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "No gpg key found with the given key-id");
+                   "No gpg key found with ID %s (homedir: %s)", key_id,
+                   homedir ? homedir : "<default>");
       goto out;
     }
   
@@ -1666,43 +1697,22 @@ out:
 #endif
 }
 
-/**
- * ostree_repo_verify_commit:
- * @self: Repository
- * @commit_checksum: ASCII SHA256 checksum
- * @keyringdir: (allow-none): Path to directory GPG keyrings; overrides built-in default if given
- * @extra_keyring: (allow-none): Path to additional keyring file (not a directory)
- * @cancellable: Cancellable
- * @error: Error
- *
- * Check for a valid GPG signature on commit named by the ASCII
- * checksum @commit_checksum.
- */
-gboolean
-ostree_repo_verify_commit (OstreeRepo   *self,
-                           const gchar  *commit_checksum,
-                           GFile        *keyringdir,
-                           GFile        *extra_keyring,
-                           GCancellable *cancellable,
-                           GError      **error)
+static gboolean
+_ostree_repo_gpg_verify_file_with_metadata (OstreeRepo          *self,
+                                            GFile               *path,
+                                            GVariant            *metadata,
+                                            GFile               *keyringdir,
+                                            GFile               *extra_keyring,
+                                            GCancellable        *cancellable,
+                                            GError             **error)
 {
 #ifdef HAVE_GPGME
   gboolean ret = FALSE;
   gs_unref_object OstreeGpgVerifier *verifier = NULL;
-  gs_unref_variant GVariant *commit_variant = NULL;
-  gs_unref_object GFile *commit_tmp_path = NULL;
-  gs_unref_object GFile *keyringdir_ref = NULL;
-  gs_unref_variant GVariant *metadata = NULL;
   gs_unref_variant GVariant *signaturedata = NULL;
-  gs_free gchar *commit_filename = NULL;
   gint i, n;
   gboolean had_valid_signataure = FALSE;
 
-  if (!ostree_repo_load_variant (self, OSTREE_OBJECT_TYPE_COMMIT,
-                                 commit_checksum, &commit_variant,
-                                 error))
-    goto out;
-
   verifier = _ostree_gpg_verifier_new (cancellable, error);
   if (!verifier)
     goto out;
@@ -1720,16 +1730,6 @@ ostree_repo_verify_commit (OstreeRepo   *self,
         goto out;
     }
 
-  if (!ostree_repo_read_commit_detached_metadata (self,
-                                                  commit_checksum,
-                                                  &metadata,
-                                                  cancellable,
-                                                  error))
-    {
-      g_prefix_error (error, "Failed to read detached metadata: ");
-      goto out;
-    }
-  
   if (metadata)
     signaturedata = g_variant_lookup_value (metadata, "ostree.gpgsigs", G_VARIANT_TYPE ("aay"));
   if (!signaturedata)
@@ -1739,18 +1739,6 @@ ostree_repo_verify_commit (OstreeRepo   *self,
       goto out;
     }
 
-  if (!gs_file_open_in_tmpdir (self->tmp_dir, 0644,
-                               &commit_tmp_path, NULL,
-                               cancellable, error))
-    goto out;
-  
-  if (!g_file_replace_contents (commit_tmp_path,
-                                (char*)g_variant_get_data (commit_variant),
-                                g_variant_get_size (commit_variant),
-                                NULL, FALSE, 0, NULL,
-                                cancellable, error))
-    goto out;
-
   n = g_variant_n_children (signaturedata);
   for (i = 0; i < n; i++)
     {
@@ -1770,7 +1758,7 @@ ostree_repo_verify_commit (OstreeRepo   *self,
         goto out;
 
       if (!_ostree_gpg_verifier_check_signature (verifier,
-                                                 commit_tmp_path,
+                                                 path,
                                                  temp_sig_path,
                                                  &had_valid_signataure,
                                                  cancellable, error))
@@ -1789,11 +1777,9 @@ ostree_repo_verify_commit (OstreeRepo   *self,
                    "GPG signatures found, but none are in trusted keyring");
       goto out;
     }
-  
+
   ret = TRUE;
-out:
-  if (commit_tmp_path)
-    (void) gs_file_unlink (commit_tmp_path, NULL, NULL);
+ out:
   return ret;
 #else
   g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
@@ -1801,3 +1787,70 @@ out:
   return FALSE;
 #endif
 }
+
+/**
+ * ostree_repo_verify_commit:
+ * @self: Repository
+ * @commit_checksum: ASCII SHA256 checksum
+ * @keyringdir: (allow-none): Path to directory GPG keyrings; overrides built-in default if given
+ * @extra_keyring: (allow-none): Path to additional keyring file (not a directory)
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Check for a valid GPG signature on commit named by the ASCII
+ * checksum @commit_checksum.
+ */
+gboolean
+ostree_repo_verify_commit (OstreeRepo   *self,
+                           const gchar  *commit_checksum,
+                           GFile        *keyringdir,
+                           GFile        *extra_keyring,
+                           GCancellable *cancellable,
+                           GError      **error)
+{
+  gboolean ret = FALSE;
+  gs_unref_variant GVariant *commit_variant = NULL;
+  gs_unref_object GFile *commit_tmp_path = NULL;
+  gs_unref_object GFile *keyringdir_ref = NULL;
+  gs_unref_variant GVariant *metadata = NULL;
+  gs_free gchar *commit_filename = NULL;
+
+  /* Create a temporary file for the commit */
+  if (!ostree_repo_load_variant (self, OSTREE_OBJECT_TYPE_COMMIT,
+                                 commit_checksum, &commit_variant,
+                                 error))
+    goto out;
+  if (!gs_file_open_in_tmpdir (self->tmp_dir, 0644,
+                               &commit_tmp_path, NULL,
+                               cancellable, error))
+    goto out;
+  if (!g_file_replace_contents (commit_tmp_path,
+                                (char*)g_variant_get_data (commit_variant),
+                                g_variant_get_size (commit_variant),
+                                NULL, FALSE, 0, NULL,
+                                cancellable, error))
+    goto out;
+
+  /* Load the metadata */
+  if (!ostree_repo_read_commit_detached_metadata (self,
+                                                  commit_checksum,
+                                                  &metadata,
+                                                  cancellable,
+                                                  error))
+    {
+      g_prefix_error (error, "Failed to read detached metadata: ");
+      goto out;
+    }
+  
+  if (!_ostree_repo_gpg_verify_file_with_metadata (self,
+                                                   commit_tmp_path, metadata,
+                                                   keyringdir, extra_keyring,
+                                                   cancellable, error))
+    goto out;
+  
+  ret = TRUE;
+out:
+  if (commit_tmp_path)
+    (void) gs_file_unlink (commit_tmp_path, NULL, NULL);
+  return ret;
+}
