@@ -30,12 +30,14 @@
 static gboolean opt_print_related;
 static char* opt_print_variant_type;
 static char* opt_print_metadata_key;
+static char* opt_print_detached_metadata_key;
 static gboolean opt_raw;
 
 static GOptionEntry options[] = {
   { "print-related", 0, 0, G_OPTION_ARG_NONE, &opt_print_related, "If given, show the \"related\" commits", NULL },
   { "print-variant-type", 0, 0, G_OPTION_ARG_STRING, &opt_print_variant_type, "If given, argument should be a filename and it will be interpreted as this type", NULL },
   { "print-metadata-key", 0, 0, G_OPTION_ARG_STRING, &opt_print_metadata_key, "Print string value of metadata key KEY for given commit", "KEY" },
+  { "print-detached-metadata-key", 0, 0, G_OPTION_ARG_STRING, &opt_print_detached_metadata_key, "Print string value of detached metadata key KEY for given commit", "KEY" },
   { "raw", 0, 0, G_OPTION_ARG_NONE, &opt_raw, "Show raw variant data" },
   { NULL }
 };
@@ -98,34 +100,53 @@ do_print_related (OstreeRepo  *repo,
 }
 
 static gboolean
-do_print_metadata_key (OstreeRepo  *repo,
-                       const char *resolved_rev,
-                       const char *key,
-                       GError **error)
+do_print_metadata_key (OstreeRepo     *repo,
+                       const char     *resolved_rev,
+                       gboolean        detached,
+                       const char     *key,
+                       GError        **error)
 {
   gboolean ret = FALSE;
-  const char *value;
+  gs_unref_variant GVariant *value = NULL;
   gs_unref_variant GVariant *commit = NULL;
   gs_unref_variant GVariant *metadata = NULL;
 
-  if (!ostree_repo_load_variant (repo, OSTREE_OBJECT_TYPE_COMMIT,
-                                 resolved_rev, &commit, error))
-    goto out;
-
-  /* PARSE OSTREE_SERIALIZED_COMMIT_VARIANT */
-  metadata = g_variant_get_child_value (commit, 1);
+  if (!detached)
+    {
+      if (!ostree_repo_load_variant (repo, OSTREE_OBJECT_TYPE_COMMIT,
+                                     resolved_rev, &commit, error))
+        goto out;
+      /* PARSE OSTREE_SERIALIZED_COMMIT_VARIANT */
+      metadata = g_variant_get_child_value (commit, 0);
+    }
+  else
+    {
+      if (!ostree_repo_read_commit_detached_metadata (repo, resolved_rev, &metadata,
+                                                      NULL, error))
+        goto out;
+      if (metadata == NULL)
+        {
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                       "No detached metadata for commit %s", resolved_rev);
+          goto out;
+        }
+    }
   
-  if (!g_variant_lookup (metadata, key, "&s", &value))
-    goto out;
+  value = g_variant_lookup_value (metadata, key, NULL);
+  if (!value)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                   "No such metadata key '%s'", key);
+      goto out;
+    }
 
-  g_print ("%s\n", value);
+  ot_dump_variant (value);
 
   ret = TRUE;
  out:
   return ret;
 }
 
-
 static gboolean
 print_object (OstreeRepo          *repo,
               OstreeObjectType     objtype,
@@ -198,12 +219,14 @@ ostree_builtin_show (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
     }
   rev = argv[1];
 
-  if (opt_print_metadata_key)
+  if (opt_print_metadata_key || opt_print_detached_metadata_key)
     {
+      gboolean detached = opt_print_detached_metadata_key != NULL;
+      const char *key = detached ? opt_print_detached_metadata_key : opt_print_metadata_key;
       if (!ostree_repo_resolve_rev (repo, rev, FALSE, &resolved_rev, error))
         goto out;
 
-      if (!do_print_metadata_key (repo, resolved_rev, opt_print_metadata_key, error))
+      if (!do_print_metadata_key (repo, resolved_rev, detached, key, error))
         goto out;
     }
   else if (opt_print_related)
@@ -224,7 +247,6 @@ ostree_builtin_show (int argc, char **argv, OstreeRepo *repo, GCancellable *canc
       gboolean found = FALSE;
       if (!ostree_validate_checksum_string (rev, NULL))
         {
-          gs_unref_variant GVariant *variant = NULL;
           if (!ostree_repo_resolve_rev (repo, rev, FALSE, &resolved_rev, error))
             goto out;
           if (!print_object (repo, OSTREE_OBJECT_TYPE_COMMIT, resolved_rev, error))
